//
//  tank
//
//  Import and Playlist Daemon for autoradio project
//
//
//  Copyright (C) 2017 Christian Pointner <equinox@helsinki.at>
//
//  This file is part of tank.
//
//  tank is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  any later version.
//
//  tank is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with tank. If not, see <http://www.gnu.org/licenses/>.
//

package main

import (
	"fmt"
	"net"
	"os"

	"github.com/codegangsta/cli"
	"github.com/coreos/go-systemd/activation"
)

func cmdInit(c *cli.Context) error {
	// TODO: implement this
	return cli.NewExitError(fmt.Sprintf("not yet implemented"), -2)
}

func cmdRun(c *cli.Context) error {
	ln, err := net.Listen("tcp", c.String("listen"))
	if err != nil {
		return cli.NewExitError(err.Error(), 1)
	}

	if err = runWeb(ln); err != nil {
		return cli.NewExitError(err.Error(), 1)
	}
	return cli.NewExitError("", 0)
}

func cmdRunSa(c *cli.Context) error {
	lns, err := activation.Listeners(true)
	if err != nil {
		return cli.NewExitError(fmt.Sprintf("fetching socket listeners from systemd failed: %s", err), 2)
	}

	fmt.Printf("got %d sockets from systemd\n", len(lns))
	if len(lns) != 1 {
		return cli.NewExitError("we need exactly one listener, shutting down.", 2)
	}

	if err = runWeb(lns[0]); err != nil {
		return cli.NewExitError(err.Error(), 1)
	}
	return cli.NewExitError("", 0)
}

func main() {
	app := cli.NewApp()
	app.Name = "tank"
	app.Version = "0.0.1"
	app.Usage = "import & playlist daemon"
	app.Flags = []cli.Flag{
		cli.StringFlag{
			Name:   "config",
			Value:  "/etc/aura/tank.yaml",
			Usage:  "path to the configuration file",
			EnvVar: "AURA_TANK_CONFIG",
		},
	}
	app.Commands = []cli.Command{
		{
			Name:   "init",
			Usage:  "initialize the store",
			Action: cmdInit,
		},
		{
			Name:  "run",
			Usage: "run the daemon",
			Flags: []cli.Flag{
				cli.StringFlag{
					Name:   "listen",
					Usage:  "address to listen on for web API",
					EnvVar: "AURA_TANK_LISTEN",
				},
			},
			Action: cmdRun,
		},
		{
			Name:   "runsa",
			Usage:  "run the daemon (using systemd socket-activation)",
			Action: cmdRunSa,
		},
	}

	app.Run(os.Args)
}
