//
//  tank
//
//  Import and Playlist Daemon for autoradio project
//
//
//  Copyright (C) 2017 Christian Pointner <equinox@helsinki.at>
//
//  This file is part of tank.
//
//  tank is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  any later version.
//
//  tank is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with tank. If not, see <http://www.gnu.org/licenses/>.
//

package store

import (
	"os"
	"time"

	"github.com/boltdb/bolt"
)

type Store struct {
	version  int
	db       *bolt.DB
	basePath string
}

//
// Initialization and Destruction
//

func openDB(dbPath string, readOnly bool) (db *bolt.DB, version int, err error) {
	if _, err = os.Stat(dbPath); err != nil {
		if os.IsNotExist(err) {
			err = nil
		}
		return
	}
	opts := &bolt.Options{Timeout: 100 * time.Millisecond, ReadOnly: readOnly}
	db, err = bolt.Open(dbPath, 0600, opts)
	if err != nil {
		return
	}

	err = db.View(func(tx *bolt.Tx) error {
		// TODO: check store version and integrity
		return nil
	})
	if err != nil {
		db.Close()
	}
	return
}

func createDB(dbPath string) (db *bolt.DB, version int, err error) {
	db, err = bolt.Open(dbPath, 0600, &bolt.Options{Timeout: 100 * time.Millisecond})
	if err != nil {
		return
	}

	err = db.Update(func(tx *bolt.Tx) error {
		// TODO: create buckets
		return nil
	})
	if err != nil {
		db.Close()
	}

	return
}

func NewStore(cfg Config) (*Store, error) {
	// TODO: check if cfg.BasePath exists (and is writeable?)

	db, version, err := openDB(cfg.DBPath, false)
	if err != nil {
		return nil, err
	}

	if db != nil {
		return &Store{version, db, cfg.BasePath}, nil
	}

	db, version, err = createDB(cfg.DBPath)
	if err != nil {
		return nil, err
	}
	return &Store{version, db, cfg.BasePath}, nil
}

func (st *Store) Close() {
	st.db.Close()
}
